<?php

/**
 * @file
 * Common functions for the Cura Childcare Suite.
 */

/**
 * Adds a new setting or updates an existing one.
 *
 * @param array $settings
 *   The settings to be saved. The array must start with type, category and subcategory indexes.
 *
 * @return cura_setting
 *   Return the setting object.
 */
function cura_settings_save($settings = array()) {
  $account = $GLOBALS['user'];
  $ip = ip_address();
  foreach ($settings as $type => $type_data) {
    foreach ($type_data as $category => $category_data) {
      foreach ($category_data as $subcategory => $subcategory_data) {
        $sql = 'SELECT setting_id FROM {cura_settings}';
        $sql .= ' WHERE type = :t AND category = :c AND subcategory = :s AND active = 1';
        $args = array(
          ':t' => $type,
          ':c' => $category,
          ':s' => $subcategory,
        );
        $setting = db_query($sql, $args)->fetchObject();
        if (empty($setting)) {
          $setting = entity_create('cura_setting', array(
            'type' => $type,
            'category' => $category,
            'subcategory' => $subcategory,
            'created_by' => $account->uid,
            'created_on' => REQUEST_TIME,
            'created_ip' => $ip,
          ));
        }
        $setting->label = $type . '->' . $category . '->' . $subcategory . ' settings';
        $setting->data = $settings[$type][$category][$subcategory];
        $setting->uid = $account->uid;
        $setting->updated_by = $account->uid;
        $setting->updated_on = REQUEST_TIME;
        $setting->updated_ip = $ip;
        $setting->active = 1;
        entity_save('cura_setting', $setting);

        // Keep an inactive copy of the settings.
        $setting->setting_id = NULL;
        $setting->active = 0;
        entity_save('cura_setting', $setting);
      }
    }
  }
}

/**
 * Gets the settings for the Cura Childcare Suite. Default settings are defined by each
 * module using hook_cura_default_settings_alter. These are then overwritten by any
 * settings defined in the cura_settings table.
 *
 * @return array
 *   An array of settings.
 */
function cura_get_settings($type = '', $category = '', $subcategory = '') {
  $default_settings = cura_get_default_settings($type, $category, $subcategory);
  $settings = array();

  $sql = 'SELECT setting_id, type, category, subcategory, data';
  $sql .= ' FROM {cura_settings}';
  $sql .= ' WHERE active = 1';
  $args = array();
  if ($type) {$sql .= ' AND type = :t'; $args[':t'] = $type;}
  if ($category) {$sql .= ' AND category = :c'; $args[':c'] = $category;}
  if ($subcategory) {$sql .= ' AND subcategory = :s'; $args[':s'] = $subcategory;}
  $sql .= ' ORDER BY type, category, subcategory';
  if ($result = db_query($sql, $args)->fetchAll()) {
    foreach ($result as $record) {
//      drupal_set_message('record<pre>' . print_r($record, TRUE) . '</pre>'); // ** TEST **
      if (empty($settings[$record->type][$record->category][$record->subcategory])) {
        $settings[$record->type][$record->category][$record->subcategory] = unserialize($record->data);
      }
      else {
        $settings[$record->type][$record->category][$record->subcategory] += unserialize($record->data);
      }
    }
  }
//  drupal_set_message('Database settings<pre>' . print_r($settings, TRUE) . '</pre>');
  $settings = drupal_array_merge_deep($default_settings, $settings);
//  drupal_set_message('After merge:<pre>' . print_r($settings, TRUE) . '</pre>');

  return $settings;
}

/**
 * Gets the default settings for the Cura Childcare Suite.
 *
 * @return array
 *   An array of settings.
 */
function cura_get_default_settings($type = NULL, $category = NULL, $subcategory = NULL) {
  $site_name = variable_get('site_name', '');
  $site_mail = explode('@', variable_get('site_mail', ini_get('sendmail_from')));
  $org_name = variable_get('cura_org_name', $site_name);
  if (empty($org_name)) $org_name = $site_name;
  $org_name_short = variable_get('cura_org_name_short', explode(' ', trim($org_name))[0]);
  if (empty($org_name_short)) $org_name_short = explode(' ', trim($org_name))[0];
  $org_type = variable_get('cura_org_type', 'Charity');
  if (empty($org_type)) $org_type = 'Charity';
  $org_number = variable_get('cura_org_number', '');
  $org_number_ofsted = variable_get('cura_org_number_ofsted', '');

  // Set default email addresses.
  $prefix = empty($org_name_short) ? '' : $org_name_short . ' ';
  $from_email = $site_name . ' <no-reply@' . $site_mail[1] . '>';
  $admin_email = $prefix . 'Administrator <admin@' . $site_mail[1] . '>';
  $admissions_email = $prefix . 'Admissions <admissions@' . $site_mail[1] . '>';
  $sup_email = $prefix . 'Supervisor <supervisor@' . $site_mail[1] . '>';

  // General default settings.
  $settings['general']['org']['general']['org_name'] = $org_name;
  $settings['general']['org']['general']['org_name_short'] = $org_name_short;
  $settings['general']['org']['general']['org_type'] = $org_type;
  $settings['general']['org']['general']['org_number'] = $org_number;
  $settings['general']['org']['general']['org_number_ofsted'] = $org_number_ofsted;

  $html = '<table style="width:100%;">' . "\n";
  $html .= "<tr>\n";
  $html .= '<td style="text-align:center;padding:20px;"><small>@org_name<br>' . "\n";
  $html .= '<a href="' . file_create_url('about') . '">About us</a> | ';
  $html .= '<a href="' . file_create_url('policy/privacy') . '">Privacy</a></small>' . "\n";
  $html .= '<td style="text-align:right;padding:20px;"><a href="http://www.facebook.com/">';
  $html .= '<img src="' . file_create_url('sites/default/files/facebook-icon_240x240.png') . '"';
  $html .= ' width="40px"></a>' . "\n";
  $html .= "</table>";
  $settings['general']['org']['general']['footer'] = $html;

  // Organisation bank details
  $settings['general']['org']['bank']['account_owner'] = $site_name;
  $settings['general']['org']['bank']['account_number'] = '';
  $settings['general']['org']['bank']['account_iban'] = '';
  $settings['general']['org']['bank']['bank_code_appellation'] = 'Sort code';
  $settings['general']['org']['bank']['bank_code'] = '';
  $settings['general']['org']['bank']['account_swift'] = '';
  $settings['general']['org']['bank']['account_bank'] = '';
  $settings['general']['org']['bank']['account_branch'] = '';
  $settings['general']['org']['bank']['instructions'] = '';

  // Calendar default settings.
  foreach (array('Mon' => 1, 'Tue' => 1, 'Wed' => 1, 'Thu' => 1, 'Fri' => 1, 'Sat' => 0, 'Sun' => 0) as $dow => $value) {
    $settings['general']['calendar']['days'][$dow]['enabled'] = $value;
    $settings['general']['calendar']['days'][$dow]['start'] = '09:00';
    $settings['general']['calendar']['days'][$dow]['finish'] = '17:00';
  }
  $settings['general']['calendar']['terms']['terms_only'] = 1;
  $settings['general']['calendar']['weeks']['weeks_open'] = 38;

  // Notification default settings.
  $settings['general']['notification']['email']['enabled'] = 1;
  $settings['general']['notification']['text']['enabled'] = 0;

  // Email default settings.
  $settings['notification']['email']['weekly_plan'] = array(
    'enabled' => 1,
    'from' => $from_email,
    'to' => $sup_email . ', ' . $admissions_email,
    'cc' => $admin_email,
  );

  $settings['notification']['email']['starters_leavers'] = array(
    'enabled' => 1,
    'from' => $from_email,
    'to' => $sup_email . ', ' . $admissions_email,
    'cc' => '',
    'count' => 60,
  );

  // Calling all modules implementing hook_cura_default_settings_alter():
  drupal_alter('cura_default_settings', $settings);

//  drupal_set_message('<h2>Default settings:</h2><pre>' . print_r($settings, TRUE) . '</pre>');

  // Only return the requested settings.
  if ($type) $settings = array($type => $settings[$type]);
  if ($category) $settings = array($type => array($category => $settings[$type][$category]));
  if ($subcategory) $settings = array($type => array($category => array($subcategory => $settings[$type][$category][$subcategory])));

  return $settings;
}

/**
 * Finds the correct ISO country details for the given string.
 *
 * @param string $string
 *   The search string. Can be an ISO country code or name.
 *
 * @return array $country
 *   The name of the country keyed by its 2-letter ISO country code.
 */
function cura_country($string) {
  $strip = array('the republic of ', 'the kingdom of ');
  $string = str_replace($strip, '', strtolower($string));
  include_once DRUPAL_ROOT . '/includes/locale.inc';
  $countries = country_get_list();
  foreach ($countries as $code => $name) {
    $is_code = preg_replace('/[^A-Z]/', '', strtoupper($string)) === preg_replace('/[^A-Z]/', '', strtoupper($code));
    $is_name = strtolower($string) === str_replace($strip, '', strtolower($name));
    if ($is_code || $is_name) {
      return array($code => $countries[$code]);
    }
  }

  // Handle alternate names.
  $country_names = array(
    'GB' => array('greatbritain'),
    'NL' => array('holland'),
    'RU' => array('russianfederation'),
    'US' => array('america', 'unitedstatesofamerica', 'usa'),
  );
  foreach ($country_names as $code => $names) {
    if (in_array(preg_replace('/[^a-z]/', '', strtolower($string)), $names)) {
      return array($code => $countries[$code]);
    }
  }

  // Default to the system setting.
  $code = variable_get('site_default_country', 'GB');

  return array($code => $countries[$code]);
}

/**
 * Gives the difference between two dates.
 *
 * @param timestamp $ts1
 *   The start UNIX timestamp.
 * @param timestamp $ts2
 *   The stop UNIX timestamp.
 * @param string $format
 *   The format in which to return the time difference (default days).
 *
 * @return string $string
 *   The formatted string from date_diff.
 */
function cura_date_diff($ts1 = REQUEST_TIME, $ts2 = NULL, $format = '%a') {
  $string = FALSE;
  if (!empty($ts2) && ($ts2 >= $ts1)) {
    $dt1 = date_create("@$ts1");
    $dt2 = date_create("@$ts2");
    $diff = date_diff($dt1, $dt2);
    $string = $diff->format($format);
  }

  return $string;
}

/**
 * Capitalises the string, including words separated by hyphens.
 *
 * @param string $string
 *   The string to capitalise.
 *
 * @return string $string
 *   Original string with only first letters capitalised.
 */
function cura_capitalise($string) {
  $string = ucwords(strtolower(trim($string)));
  $string = implode('-', array_map('ucfirst', explode('-', $string)));

  return $string;
}

/**
 * Returns of list of languages indexed by ISO 639-1 two letter language code.
 *
 * @return array $language_list
 *   Return the language list.
 */
function cura_language_list() {
  $language_list = array();
  include_once DRUPAL_ROOT . '/includes/iso.inc';
  $languages = _locale_get_predefined_list();
  foreach ($languages as $language_code => $language) {
    $language_list[$language_code] = $language[0];
  }
  if (isset($language_list['xx-lolspeak'])) unset($language_list['xx-lolspeak']);
  if (isset($language_list['en-gb'])) unset($language_list['en-gb']);
  asort($language_list);

  return $language_list;
}

/**
 * Returns an option list of vocabulary terms.
 *
 * @param string $name
 *   The machine name of the vocabulary.
 *
 * @return array $options
 *   Return the options list.
 */
function cura_taxonomy_options($name, $index = 'id', $options = array()) {
  if ($vocab = taxonomy_vocabulary_machine_name_load($name)) {
    if ($tree = taxonomy_get_tree($vocab->vid)) {
      foreach ($tree as $item) {
        if ($index === 'id') {
          $options[$item->tid] = str_repeat('-', $item->depth) . $item->name;
        }
        else {
          $options[$item->name] = $item->name;
        }
      }
    }
  }

  return $options;
}

/**
 * Adds a new currency rate record or updates an existing one.
 *
 * @param array $values
 *   The currency rate values to be saved.
 * 
 * @return cura_currency_rate
 *   Return the currency rate object.
 */
function cura_currency_rate_save($values) {
  $account = $GLOBALS['user'];
  $ip = ip_address();

  // If the record exists, get it from the database.
  $sql = 'SELECT currency_rate_id FROM {cura_currency_rates}';
  $sql .= ' WHERE currency_code = :cc AND base_code = :bc AND date = :date AND active = 1';
  $vars = array(
    ':cc' => $values['currency_code'],
    ':bc' => $values['base_code'],
    ':date' => $values['date'],
  );
  $currency_rate = db_query($sql, $vars)->fetchObject();
  if (empty($currency_rate)) {
    $currency_rate = entity_create('cura_currency_rate', array(
      'currency_code' => $values['currency_code'],
      'base_code' => $values['base_code'],
      'date' => $values['date'],
      'created_by' => $account->uid,
      'created_on' => REQUEST_TIME,
      'created_ip' => $ip,
    ));
  }
  $currency_rate->rate = round($values['rate'], 6);
  $currency_rate->updated_by = $account->uid;
  $currency_rate->updated_on = REQUEST_TIME;
  $currency_rate->updated_ip = $ip;
  $currency_rate->active = 1;
  entity_save('cura_currency_rate', $currency_rate);
} 

/**
 * Returns gender-specific words.
 *
 * @param string $gender
 *   Specifies the gender for which the words apply.
 *
 * @return array
 *   Array containing the words.
 */
function _cura_gender_words($gender) {
  if (in_array(strtolower($gender), array('m', 'male', 'he', 'him', 'boy', 'son'))) {
    return array(
      'he_she' => 'he', 'his_her' => 'his', 'him_her' => 'him',
      'boy_girl' => 'boy', 'son_daughter' => 'son',
    );
  }
  else {
    return array(
      'he_she' => 'she', 'his_her' => 'her', 'him_her' => 'her',
      'boy_girl' => 'girl', 'son_daughter' => 'daughter',
    );
  }
}

/**
 * Formats the telephone number.
 *
 * @param string $number
 *   The number string to format.
 *
 * @return string
 *   Returns the formatted telephone number string.
 */
function cura_telephone_format($number, $country = 'GB') {
  switch ($country) {
    case 'GB':
      $format_options = array (
        '02' => '3, 4, 4',
        '03' => '4, 3, 4',
        '05' => '3, 4, 4',
        '0500' => '4, 6',
        '07' => '5, 3, 3',
        '070' => '3, 4, 4',
        '076' => '3, 4, 4',
        '07624' => '5, 6',
        '08' => '4, 3, 4', // some 0800 numbers are 4, 6
        '09' => '4, 3, 4',
        '01' => '5, 3, 3', // some 01 numbers are 5, 5
        '011' => '4, 3, 4',
        '0121' => '4, 3, 4',
        '0131' => '4, 3, 4',
        '0141' => '4, 3, 4',
        '0151' => '4, 3, 4',
        '0161' => '4, 3, 4',
        '01865' => '5, 3, 3',
        '0191' => '4, 3, 4',
        '013873' => '6, 5',
        '015242' => '6, 5',
        '015394' => '6, 5',
        '015395' => '6, 5',
        '015396' => '6, 5',
        '016973' => '6, 5',
        '016974' => '6, 5',
        '016977' => '6, 5',
        '0169772' => '6, 4',
        '0169773' => '6, 4',
        '017683' => '6, 5',
        '017684' => '6, 5',
        '017687' => '6, 5',
        '019467' => '6, 5',
      );
      uksort($format_options, function($a, $b) { // Sort by length of telephone code
        return strlen($b) - strlen($a);
      });

      // Change the international number format and remove any non-number character
      $new_number = preg_replace('/^\+/', '00', $number);
      $new_number = preg_replace('/[^0-9]+/', '', $new_number);
      $new_number = preg_replace('/^00440/', '0', $new_number);
      $new_number = preg_replace('/^0044/', '0', $new_number);

      foreach ($format_options AS $key => $format) {
        if (substr($new_number, 0, strlen($key)) == $key) break;
      }
      $format = explode(',', $format); // Convert to array
      $start = 0;
      $array = array();
      foreach ($format AS $value) {
        $array[] = substr($new_number, $start, $value);
        $start = $start + $value;
      }
      $number = preg_replace('/^0/', '+44 ', implode(' ', $array));
      break;
  }

  return $number;
}

/**
 * Provides the standard email signature.
 *
 * @param string $name
 *   The name to include in the signature.
 *
 * @return string
 *   Returns a signature as an HTML formatted string.
 */
function cura_email_signature($name = NULL) {
  $org_name = variable_get('cura_org_name', '?');
  $org_type = variable_get('cura_org_type', '?');
  $org_number = variable_get('cura_org_number');
  $url = file_create_url('');
  $scheme = parse_url($url);
  $html = "";
  if (!empty($name)) $html .= "<p>Kind regards<br>\n" . check_plain($name) . "</p>\n\n";
  $html .= "<p><b>" . $org_name;
  $html .= $org_number ? " (" . $org_type . " number " . $org_number . ")<br>\n" : "";
  $html .= '<a href="' . $url . '">' . $scheme['host'] . "</a></b></p>";

  return $html;
}

/**
 * Gets the currency exchange rates as at end of yesterday.
 *
 * @return array
 *   Returns an array of currency exchange rates keyed by currency code.
 */
function cura_get_exchange_rates($base = NULL) {
  $base = !empty($base) ? preg_replace('/[^A-Z]/', '', strtoupper($base)) : variable_get('commerce_default_currency', 'GBP');
  $url = 'https://openexchangerates.org/api/historical/';
  $url .= date('Y-m-d', strtotime('yesterday')) . '.json';
  $url .= '?app_id=0da823297cf54efd871e02fde60c7632';
//  $url .= '&base=' . $base; // Cannot change base currency with free plan
  $request = drupal_http_request($url);
  drupal_set_message('<pre>url:<br>' . $url . '</pre>');
  drupal_set_message('<pre>request:<br>' . print_r($request, TRUE) . '<br></pre>'); // ** TESTING **
  if ($request) {
    $request = drupal_json_decode($request->data);
    if (isset($request['rates'][$base])) {
      $dt = new DateTime("now", new DateTimeZone('UTC'));
      $dt->setTimestamp($request['timestamp']);
      $request['base'] = $base;
      $base_rate = $request['rates'][$base];
      // Convert default USD rates to specified base currency.
      foreach ($request['rates'] as $code => $rate) {
        $request['rates'][$code] = $rate / $base_rate;
        cura_currency_rate_save(array(
          'currency_code' => $code, 'base_code' => $base, 'date' => $request['timestamp'], 'rate' => $request['rates'][$code],
        ));
      }
      watchdog('cura', 'Currency exchange rates retrieved from @url with timestamp @ts.', array('@url' => $url, '@ts' => $dt->format('H:i:s l, j M Y')), WATCHDOG_INFO);
    }
    else {
      watchdog('cura', 'No exchange rate for base currency was provided.', NULL, WATCHDOG_ERROR);
    }
  }
  else {
    watchdog('cura', 'Error retrieving open exchange rates.', NULL, WATCHDOG_ERROR);
  }

  return $request;
}

/**
 * Gets the school calendar for the specified period.
 *
 * @param int $start
 *   The first day of the period (Unix timestamp).
 * @param int $finish
 *   The last day of the period (Unix timestamp).
 *
 * @return array|FALSE
 *   Returns the school calendar or FALSE if period length <= 0.
 */
function cura_get_calendar($start = REQUEST_TIME, $finish = NULL) {
  $calendar = array();
  // Default 7 days.
  if (is_null($finish)) $finish = $start + 518400;
  $calendar['start'] = date('Y-m-d', $start);
  $calendar['finish'] = date('Y-m-d', $finish);
  // Validate start and finish dates.
  if ($calendar['finish'] <= $calendar['start']) {
    return FALSE;
  }
  else {
    $calendar['description'] = 'Calendar for ';
    $calendar['description'] .= date('D, j M ', $start);
    if (date('Y', $start) != date('Y', $finish)) $calendar['description'] .= date('Y ', $start);
    $calendar['description'] .= 'to ' . date('D, j M Y', $finish);
    for ($day = $calendar['start']; $day <= $calendar['finish']; $day = date('Y-m-d', strtotime('+1 day' . $day))) {
      $calendar['days'][$day]['billable'] = 1;
      $calendar['days'][$day]['hours_n'] = $calendar['days'][$day]['hours_f'] = 0;
    }
    cura_calendar_terms_update($calendar);
    cura_calendar_holidays_update($calendar);
    cura_calendar_closures_update($calendar);
  }
//  drupal_set_message('<pre>calendar:<br>'.print_r($calendar, TRUE).'<br></pre>'); // ** TESTING **

  return $calendar;
}

/**
 * Updates the school term information on the calendar.
 */
function cura_calendar_terms_update(&$calendar) {
  // Convert the dates to Unix timestamps.
  $start = strtotime($calendar['start']);
  $finish = strtotime($calendar['finish']);
  if ($terms = cura_get_school_terms_in_period($start, $finish)) {
    foreach ($terms as $term) {
      $tw = entity_metadata_wrapper('node', $term);
      $term_start = date('Y-m-d', $tw->field_term_dates->value->value());
      $term_finish = date('Y-m-d', $tw->field_term_dates->value2->value());
      $calendar['terms'][$term->title] = array(
        'start' => date('j M Y', strtotime($term_start)),
        'finish' => date('j M Y', strtotime($term_finish)),
      );
      $term_titles[] = $term->title . ': ' . $calendar['terms'][$term->title]['start']
                       . ' to ' . $calendar['terms'][$term->title]['finish'];
      for ($day = $term_start; $day <= $term_finish; $day = date('Y-m-d', strtotime('+1 day' . $day))) {
        if ($day >= $calendar['start'] && $day <= $calendar['finish']) {
          $calendar['days'][$day]['term'] = $term->title;
        }
      }
    }
    $calendar['description'] = implode("\r\n", $term_titles);
  }
}

/**
 * Updates the public and school holiday information on the calendar.
 */
function cura_calendar_holidays_update(&$calendar) {
  // Convert the dates to Unix timestamps.
  $start = strtotime($calendar['start']);
  $finish = strtotime($calendar['finish']);
  $billable = variable_get('cura_bill_public_holidays', '0');
  if ($holidays = cura_get_holidays($start, $finish)) {
    foreach ($holidays as $day => $holiday) {
      // Convert Ymd to Y-m-d, alter get_holidays function later.
      $d = date('Y-m-d', strtotime($day));
      $calendar['days'][$d]['billable'] = $billable;
      $calendar['days'][$d]['holiday'] = $holiday['title'];
    }
  }
  $billable = variable_get('cura_bill_school_holidays', '0');
  if ($school_holidays = cura_get_school_holidays($start, $finish)) {
    foreach ($school_holidays as $day => $school_holiday) {
      // Convert Ymd to Y-m-d, alter function later.
      $d = date('Y-m-d', strtotime($day));
      $calendar['days'][$d]['billable'] = $billable;
      $calendar['days'][$d]['term'] = 'School holiday';
    }
  }
}

/**
 * Updates the school closure information on the calendar.
 */
function cura_calendar_closures_update(&$calendar) {
  // Convert the dates to Unix timestamps.
  $start = strtotime($calendar['start']);
  $finish = strtotime($calendar['finish']);
  $billable = variable_get('cura_bill_closures', '0');
  if ($closures = cura_get_closures($start, $finish)) {
    foreach ($closures as $day => $closure) {
      // Convert Ymd to Y-m-d, alter get_closures function later.
      $d = date('Y-m-d', strtotime($day));
      $calendar['days'][$d]['billable'] = $billable;
      $calendar['days'][$d]['closure'] = $closure['title'];
    }
  }
}

/**
 * Gets the school term for the specified date.
 *
 * @param int $date
 *   The date to identify the school term (Unix timestamp).
 *
 * @return object|FALSE
 *   Returns the current school term or FALSE if date not in term.
 */
function cura_get_current_school_term($date = REQUEST_TIME) {
  $term = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'school_term')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_term_dates', 'value', date('Y-m-d', $date), '<=')
    ->fieldCondition('field_term_dates', 'value2', date('Y-m-d', $date), '>=')
    ->fieldOrderBy('field_term_dates', 'value', 'DESC')
    ->range(0, 1) // We want only one school term
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $term_nid = current(array_keys($result['node']));
    $term = node_load($term_nid);
  }

  return $term;
}

/**
 * Get a list of active school terms for specified period.
 *
 * @param int $start
 *   The start of the period (Unix timestamp).
 * @param int $finish
 *   The finish of the period (Unix timestamp).
 *
 * @return object|FALSE
 *   Returns the school terms in the period or FALSE if no terms found.
 */
function cura_get_school_terms_in_period($start, $finish) {
  $terms = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'school_term')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_term_dates', 'value', date('Y-m-d', $finish), '<=')
    ->fieldCondition('field_term_dates', 'value2', date('Y-m-d', $start), '>=')
    ->fieldOrderBy('field_term_dates', 'value', 'ASC')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $term_nids = array_keys($result['node']);
    $terms = entity_load('node', $term_nids);
  }

  return $terms;
}

/**
 * Get a list of active school terms.
 *
 * @param string $type
 *   The type of result to return.
 * @param int $date
 *   The date to determine the first school term (Unix timestamp).
 * @param int $count
 *   The number of school terms to retrieve.
 */
function cura_get_school_terms($type = 'nids', $date = REQUEST_TIME, $count = 1) {
  if ($count <= 0) return FALSE;
  if ($count == 1) {
    if ($current_term = cura_get_current_school_term($date)) { // Date falls in a term
      $date = strtotime($current_term->field_term_dates[LANGUAGE_NONE][0]['value']);
    }
  }
  else {
    $query = new EntityFieldQuery();
    $query
      ->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', 'school_term')
      ->propertyCondition('status', 1)
      ->fieldCondition('field_term_dates', 'value', date('Y-m-d', $date), '<')
      ->fieldOrderBy('field_term_dates', 'value', 'DESC')
      ->range(0, ceil($count/2)) // We want the starting school term
      // Run the query as user 1.
      ->addMetaData('account', user_load(1));
    $result = $query->execute();
    if (isset($result['node'])) {
      $term_nids = array_keys($result['node']);
      $term = node_load($term_nids[count($term_nids)-1]);
      $date = strtotime($term->field_term_dates[LANGUAGE_NONE][0]['value']);
    }
  }

  $return = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'school_term')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_term_dates', 'value', date('Y-m-d', $date), '>=')
    ->fieldOrderBy('field_term_dates', 'value', 'ASC')
    ->range(0, $count)
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $term_nids = array_keys($result['node']);
    switch ($type) {
      case 'nids':
        $return = $term_nids;
        break;
      case 'objects':
        $return = entity_load('node', $term_nids);
//        drupal_set_message('<pre>terms:<br>'.print_r($return, TRUE).'<br>'); // ** TESTING **
        break;
      case 'options':
        $terms = entity_load('node', $term_nids);
        foreach ($terms as $nid => $term) {
          $tw = entity_metadata_wrapper('node', $term);
          $return[$nid] = $tw->title->value().' ('
                          .date('Y-m-d', $tw->field_term_dates->value->value()).' - '
                          .date('Y-m-d', $tw->field_term_dates->value2->value()).')';
        }
        break;
      case 'tableselect':
        $terms = entity_load('node', $term_nids);
        foreach ($terms as $nid => $term) {
          $tw = entity_metadata_wrapper('node', $term);
          $return[$nid] = array(
            'title' => $tw->title->value(),
            'start' => date('Y-m-d', $tw->field_term_dates->value->value()),
            'finish' => date('Y-m-d', $tw->field_term_dates->value2->value()),
          );
        }
        break;
    }
  }

  return $return;
}

/**
 * Get the bank (public/state) holidays for the period specified.
 */
function cura_get_holidays($start, $finish) {
  $tz = date_default_timezone_get();
  date_default_timezone_set('UTC');
  $holidays = array();
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'holiday')
    ->propertyCondition('status', 1)
    ->fieldOrderBy('field_calendar_date', 'value', 'ASC')
    ->fieldCondition('field_calendar_date', 'value',
        array(date('Y-m-d H:i:s', $start), date('Y-m-d H:i:s', $finish)), 'BETWEEN')
    // Exclude school holidays.
    ->fieldCondition('field_holiday_type', 'tid', 10, '<>')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $holiday_nids = array_keys($result['node']);
    $holiday_nodes = node_load_multiple($holiday_nids);
    foreach ($holiday_nodes as $holiday_key=>$holiday_node) {
      $holiday_date = strtotime($holiday_node->field_calendar_date[LANGUAGE_NONE][0]['value']);
      $holiday_timezone = $holiday_node->field_calendar_date[LANGUAGE_NONE][0]['timezone'];
      $holidays[format_date($holiday_date, 'custom', 'Ymd', $holiday_timezone)] = array(
        'title'=>$holiday_node->title,
        'date'=>$holiday_date,
        'timezone'=>$holiday_timezone,
      );
    }
  }
  date_default_timezone_set($tz);

  return $holidays;
}

/**
 * Get the school holidays for the specified period.
 *
 * @param int $start
 *   The start date of the period (Unix timestamp).
 * @param int $finish
 *   The finish date of the period (Unix timestamp).
 */
function cura_get_school_holidays($start, $finish) {
  $school_holidays = FALSE;
  $terms = cura_get_school_terms_in_period($start, $finish);
//  drupal_set_message('<pre>Terms<br>'.print_r($terms, TRUE).'</pre>'); // ** TESTING **
  for ($day = date('Ymd', $start); $day <= date('Ymd', $finish); $day = date('Ymd', strtotime('+1 day' . $day))) {
    $school_holidays[$day] = 1;
    if ($terms) {
      foreach ($terms as $term) {
        $tw = entity_metadata_wrapper('node', $term);
        $date = $tw->field_term_dates->value();
        $term_start = format_date($tw->field_term_dates->value->value(), 'custom', 'Ymd',
                      $date['timezone']);
        $term_finish = format_date($tw->field_term_dates->value2->value(), 'custom', 'Ymd',
                       $date['timezone']);
        if ($day >= $term_start && $day <= $term_finish) { // Term day
          unset($school_holidays[$day]);
        }
      }
    }
  }

  return $school_holidays;
}

/**
 * Get the school closures for the period specified.
 */
function cura_get_closures($start, $finish) {
  $tz = date_default_timezone_get();
  date_default_timezone_set('UTC');
  $closures = array();
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'closure')
    ->propertyCondition('status', 1)
    ->fieldOrderBy('field_calendar_date', 'value', 'ASC')
    ->fieldCondition('field_calendar_date', 'value',
        array(date('Y-m-d H:i:s', $start), date('Y-m-d H:i:s', $finish)), 'BETWEEN')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $closure_nids = array_keys($result['node']);
    $closure_nodes = node_load_multiple($closure_nids);
    foreach ($closure_nodes as $closure_key=>$closure_node) {
      $closure_date = strtotime($closure_node->field_calendar_date[LANGUAGE_NONE][0]['value']);
      $closure_timezone = $closure_node->field_calendar_date[LANGUAGE_NONE][0]['timezone'];
      $closures[format_date($closure_date, 'custom', 'Ymd', $closure_timezone)] = array(
          'title'=>$closure_node->title,
          'date'=>$closure_date,
          'timezone'=>$closure_timezone,
      );
    }
  }
  date_default_timezone_set($tz);

  return $closures;
}

/**
 * Get the date from which funding starts for the given birth date.
 *
 * @param int $birth_date
 *   The birth date of the child (Unix timestamp).
 * @param string $type
 *   The type of funding.
 *
 * @return int
 *   The date from which the funding starts.
 */
function cura_get_funding_start($birth_date, $type = '3') {
  $date = strtotime('+' . $type . ' years', $birth_date);
  switch (true) {
    case (date('n', $date) < 4):
      $date = strtotime('1 Apr', $date);
      break;
    case (date('n', $date) < 9):
      $date = strtotime('1 Sep', $date);
      break;
    default:
      $date = strtotime('+1 year 1 Jan', $date);
      break;
  }

  return $date;
}

/**
 * Get a list of active children.
 *
 * @param string $type
 *   The type of result to return
 * @param string $status
 *   Restrict the list to children of a certain status
 */
function cura_get_children($type = 'select', $statuses = array('N', 'W', 'P'), $uid = NULL) {
  $return = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'child')
//    ->propertyCondition('status', 1)
    ->fieldOrderBy('cura_lastname', 'value', 'ASC')
    ->fieldOrderBy('cura_firstname', 'value', 'ASC')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  if (!empty($uid)) {
    if ($person = cura_person_load_by_uid($uid)) {
      $account = cura_get_account_by_person($person->nid);
      $query->fieldCondition('field_account_id', 'target_id', $account->nid);
    }
    else {
      return FALSE;
    }
  }
  if ($statuses <> 'All') {
    $query->fieldCondition('field_child_status', 'value', $statuses);
  }
  $result = $query->execute();
  if (isset($result['node'])) { // Child record exists
    $child_nids = array_keys($result['node']);
    switch ($type) {
      case 'nids':
        $return = $child_nids;
        break;
      case 'objects':
        $return = entity_load('node', $child_nids);
        break;
      case 'select':
        $children = entity_load('node', $child_nids);
        foreach ($children as $nid => $child) {
          $cw = entity_metadata_wrapper('node', $child);
          $return[$nid] = $cw->cura_lastname->value().', '
                          .$cw->cura_firstname->value().' ('
                          .date('Y-m-d', strtotime($child->field_birth_date[LANGUAGE_NONE][0]['value'])).')';
        }
        break;
      case 'tableselect':
        $children = entity_load('node', $child_nids);
        foreach ($children as $nid => $child) {
          $cw = entity_metadata_wrapper('node', $child);
          $return[$nid] = array(
            'name' => $cw->cura_lastname->value().', '.$cw->cura_firstname->value(),
            'birthdate' => date('Y-m-d', strtotime($child->field_birth_date[LANGUAGE_NONE][0]['value'])),
          );
        }
        break;
      case 'tableselect2':
        $children = entity_load('node', $child_nids);
        foreach ($children as $nid => $child) {
          $cw = entity_metadata_wrapper('node', $child);
          $return[$nid] = array( // For tableselect
            'name' => $cw->cura_lastname->value().', '.$cw->cura_firstname->value(),
            'birthdate' => date('Y-m-d', strtotime($child->field_birth_date[LANGUAGE_NONE][0]['value'])),
            'status' => $cw->field_child_status->value(),
          );
        }
        break;
    }
//    drupal_set_message('<pre>options:<br>'.print_r($options, TRUE).'<br>'); // ** TESTING **
  }

  return $return;
}

/**
 * Get the start date for the specified child ID.
 *
 * @param int $child_id
 *   The ID of the child.
 *
 * @return int|FALSE
 */
function cura_get_child_start_date($child_id) {
  $start_date = db_query('SELECT start FROM {cura_session_plans} WHERE child_id = :child_id'
                         .' ORDER BY start',
                         array(':child_id' => $child_id))->fetchField();
//  $first_session_date = db_query('SELECT date FROM {cura_sessions} WHERE child_id = :child_id'
//                         .' ORDER BY date',
//                         array(':child_id' => $child_id))->fetchField();
//  drupal_set_message("Start: ".$start_date." (".date('Y-m-d H:i:s', $start_date)
//                     ."), First session: ".$first_session_date." (".date('Y-m-d H:i:s', $first_session_date).")");
//  if (empty($start_date) || !empty($first_session_date) && $start_date > $first_session_date) $start_date = $first_session_date;

  return empty($start_date) ? false : $start_date;
}

/**
 * Get the person for the specified user ID.
 *
 * @param int $uid
 *   The ID of the user.
 *
 * @return node|FALSE
 */
function cura_person_load_by_uid($uid) {
  $person = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'person')
    ->propertyCondition('status', 1)
    ->propertyOrderBy('created', 'DESC')
    ->fieldCondition('field_user_id', 'target_id', $uid, '=')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $person_nids = array_keys($result['node']);
    $person = node_load($person_nids[0]);
  }

  return $person;
}

/**
 * Get the account for the specified person ID.
 *
 * @param int $person_id
 *   The ID of the person.
 *
 * @return node|FALSE
 */
function cura_get_account_by_person($person_id) {
  $account = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'account')
    ->propertyCondition('status', 1)
    ->propertyOrderBy('created', 'DESC')
    ->fieldCondition('field_account_owner', 'target_id', $person_id, '=')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $account_nids = array_keys($result['node']);
    $account = node_load($account_nids[0]);
  }

  return $account;
}

/**
 * Get staff email addresses.
 *
 * @param int $?
 *   The ...
 * @param int[] $child_nids
 *   The ...
 *
 * @return cura_session_plan[]|FALSE
 *   Session plan objects or FALSE if no results.
 */
function cura_staff_get_email_addresses() {
  $emails = '';

  return $emails;
}

/**
 * Get a waiting list position.
 */
function cura_get_waiting_list_position($child) {
  $position = 1;
  if (is_numeric($child)) $child = node_load($child);
  if ($child && $child->type === 'child') {
    $start_date = date('Y-m-d', strtotime($child->field_child_start_date[LANGUAGE_NONE][0]['value']));
    $query = new EntityFieldQuery();
    $query
      ->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', 'child')
      ->propertyCondition('status', 1)
      ->fieldCondition('field_child_status', 'value', 'W')
      ->fieldCondition('field_child_start_date', 'value', $start_date, '<=')
      ->fieldOrderBy('field_child_start_date', 'value', 'DESC')
      ->fieldOrderBy('field_child_wl_position', 'value', 'DESC')
      ->range(0, 1)
      ->addMetaData('account', user_load(1));
    $result = $query->execute();
    if (isset($result['node'])) {
      $child_before = node_load(current(array_keys($result['node'])));
      $position = $child_before->field_child_wl_position[LANGUAGE_NONE][0]['value'] + 1;
    }
  }

  return $position;
}

/**  
 * Update the positions on the waiting list.
 */
function cura_waiting_list_position_update() {
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'child')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_child_status', 'value', 'W')
    ->fieldOrderBy('field_child_wl_position', 'value', 'ASC')
    ->fieldOrderBy('field_child_start_date', 'value', 'ASC')
    ->propertyOrderBy('nid', 'ASC')
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  $child_nids = array_keys($result['node']);
  $children = node_load_multiple($child_nids);
  $new_position = 1;
  if ($children) {
    foreach ($children as $child) {
      $child->field_child_wl_position[LANGUAGE_NONE][0]['value'] = $new_position;
      node_save($child);
      $new_position++;
    }
    watchdog('cura', 'Updated positions for @count children on the waiting list.', array('@count' => count($children)));
  }
  else {
    watchdog('cura', 'Cannot update positions as there are no children on the waiting list.');
  }
}

/**
 * Load the session plans for the specified children and time-frame.
 *
 * @param int $start
 *   The start date of the period (Unix timestamp).
 * @param int $finish
 *   The finish date of the period (Unix timestamp).
 * @param int[] $child_nids
 *   IDs of the children.
 *
 * @return cura_session_plan[]|FALSE
 *   Session plan objects or FALSE if no results.
 */
function cura_session_plan_load($start, $finish, $child_nids) {
  $session_plans = FALSE;
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'cura_session_plan')
    ->propertyCondition('child_id', $child_nids, 'IN')
    ->propertyOrderBy('child_id', 'ASC')
    ->propertyOrderBy('session_type_id', 'ASC')
    ->propertyOrderBy('start', 'DESC')
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['cura_session_plan'])) {
    $session_plans = entity_load('cura_session_plan', array_keys($result['cura_session_plan']));
  }

  return $session_plans;
}

/**
 * Get the sessions for the specified children and time-frame.
 *
 * @param int $start
 *   The start date of the period (Unix timestamp).
 * @param int $finish
 *   The finish date of the period (Unix timestamp).
 * @param int[] $child_nids
 *   IDs of the children.
 *
 * @return cura_session[]|FALSE
 *   Session objects or FALSE if no results.
 */
function cura_get_sessions($start = NULL, $finish = NULL, $child_nids = NULL) {
  $sessions = FALSE;
  if (is_null($start)) $start = strtotime('Mon');
  if (is_null($finish)) $finish = strtotime('+6 days', $start);
  $start = strtotime(date('Y-m-d', $start));
  $finish = strtotime(date('Y-m-d', $finish));
  if (!is_null($child_nids) && !is_array($child_nids)) $child_nids = array($child_nids);
  $sql = 'SELECT s.session_id, s.date, s.session_type_id, s.session_type_label, s.start,';
  $sql .= ' s.hours, s.hours_n, s.hours_f,';
  $sql .= ' s.amount, s.amount_n, s.amount_f,';
  $sql .= ' n.nid, f.cura_firstname_value AS firstname, l.cura_lastname_value AS lastname,';
  $sql .= ' b.field_birth_date_value AS birthdate';
  $sql .= ' FROM {cura_sessions} s';
  $sql .= ' INNER JOIN {node} n ON s.child_id = n.nid';
  $sql .= ' LEFT JOIN {field_data_cura_lastname} l ON n.nid = l.entity_id';
  $sql .= ' AND (l.entity_type = \'node\' AND l.deleted = \'0\')';
  $sql .= ' LEFT JOIN {field_data_cura_firstname} f ON n.nid = f.entity_id';
  $sql .= ' AND (f.entity_type = \'node\' AND f.deleted = \'0\')';
  $sql .= ' LEFT JOIN {field_data_field_birth_date} b ON n.nid = b.entity_id';
  $sql .= ' AND (b.entity_type = \'node\' AND b.deleted = \'0\' AND b.delta = \'0\')';
  $sql .= ' WHERE s.date >= ' . $start . ' AND s.date < ' . $finish . ' + 86400';
  if (!is_null($child_nids)) $sql .= ' AND (child_id IN (:child_nids))';
  $sql .= ' ORDER BY date ASC, start ASC, lastname ASC, firstname ASC, nid ASC';
  $result = db_query($sql, array(':child_nids' => $child_nids))->fetchAll();
//  drupal_set_message('<pre>result:<br>' . print_r($result, TRUE) . '<br>'); // ** TESTING **

  return $result;
}

/**
 * Get a summary of session information for the specified children and time-frame.
 *
 * @param int $start
 *   The start date of the period (Unix timestamp).
 * @param int $finish
 *   The finish date of the period (Unix timestamp).
 * @param int[] $child_nids
 *   IDs of the children.
 *
 * @return stdObject[]|FALSE
 *   Session summary objects or FALSE if no results.
 */
function cura_get_session_summary($start = NULL, $finish = NULL, $child_nids = NULL) {
  if (is_null($start)) $start = strtotime('Mon');
  if (is_null($finish)) $finish = strtotime('+6 days', $start);
  $start = strtotime(date('Y-m-d', $start));
  $finish = strtotime(date('Y-m-d', $finish));
  if (!is_null($child_nids) && !is_array($child_nids)) $child_nids = array($child_nids);
  $sql = 'SELECT date, COUNT(DISTINCT child_id) AS children,';
  $sql .= ' SUM(hours) AS hours, SUM(hours_n) AS hours_n, SUM(hours_f) AS hours_f,';
  $sql .= ' SUM(amount) AS amount, SUM(amount_n) AS amount_n, SUM(amount_f) AS amount_f';
  $sql .= ' FROM {cura_sessions}';
  $sql .= ' WHERE (date >= ' . $start . ') AND (date < ' . $finish . ' + 86400)';
  if (!is_null($child_nids)) $sql .= ' AND (child_id IN (:child_nids))';
  $sql .= ' GROUP BY date';
  $sql .= ' ORDER BY date ASC';
  $result = db_query($sql, array(':child_nids' => $child_nids))->fetchAll();
//  drupal_set_message('<pre>result:<br>' . print_r($result, TRUE) . '<br>'); // ** TESTING **

  return $result;
}

/**
 * Load the last session for a specified session type ID and child ID.
 *
 * @param int $session_type_id
 *   The ID of the session type.
 * @param int $child_id
 *   The ID of the child.
 *
 * @return object|FALSE
 *   Returns the cura_session object or FALSE if no results.
 */
function cura_session_last_load($session_type_id, $child_id, $active = TRUE) {
  $query = new EntityFieldQuery();
  $query
    ->entityCondition('entity_type', 'cura_session')
    ->propertyCondition('session_type_id', $session_type_id)
    ->propertyCondition('child_id', $child_id)
    ->propertyOrderBy('date', 'DESC')
    ->range(0, 1)
    // Run the query as user 1.
    ->addMetaData('account', user_load(1));
  if (!empty($active)) $query->propertyCondition('active', $active);
  $result = $query->execute();
  if (isset($result['cura_session'])) {
    $session_id = array_keys($result['cura_session']);
    return current(entity_load('cura_session', $session_id));
  }

  return FALSE;
}

/**
 * Save a comment for the specified node.
 *
 * @param int $nid
 *   The identifier for the node.
 * @param string $subject
 *   The text for the subject of the comment.
 * @param string $text
 *   The comment text to save.
 */
function cura_comment_save($nid, $subject, $text) {
  if (isset($nid) && !empty($text)) {
    $account = $GLOBALS['user'];
    $comment = new stdClass(); // We create a new comment object
    $comment->nid = $nid; // nid of a node you want to attach a comment to
    $comment->cid = 0; // leave it as is
    $comment->pid = 0; // parent comment id, 0 if none
    $comment->uid = $account->uid; // id of user
    $comment->mail = $account->mail; // email of user
    $comment->is_anonymous = 0; // leave it as is
    $comment->status = COMMENT_PUBLISHED; // We auto-publish this comment
    $comment->language = LANGUAGE_NONE; // The same as for a node
    $comment->subject = $subject;
    $comment->comment_body[$comment->language][0]['value'] = $text; // Everything here is pretty much like with a node
    $comment->comment_body[$comment->language][0]['format'] = 'filtered_html';
    comment_submit($comment); // saving a comment
    comment_save($comment);
    return $comment->cid;
  }
  else {
    return 0;
  }
}

/**
 * Adds a new notification or updates an existing one.
 *
 * @param cura_notification $notification
 *   The notification object to be saved.
 *
 * @return cura_notification
 *   Return the notification object.
 */
function cura_notification_save($notification) {
  $account = $GLOBALS['user'];
  $ip = ip_address();
  if (!isset($notification->is_new)) {
    $notification->is_new = empty($notification->notification_id);
  }
  if ($notification->is_new) {
    $notification = entity_create('cura_notification', array(
      'label' => !empty($notification->label) ? $notification->label : 'Default label',
      'type' => !empty($notification->type) ? $notification->type : 'Default type',
      'notes' => !empty($notification->notes) ? $notification->notes : 'Default notes',
      'date' => !empty($notification->date) ? $notification->date : REQUEST_TIME,
      'child_id' => !empty($notification->child_id) ? $notification->child_id : NULL,
      'uid' => !empty($notification->uid) ? $notification->uid : $account->uid,
      'created_by' => $account->uid,
      'created_on' => REQUEST_TIME,
      'created_ip' => $ip,
      'active' => 1,
    ));
  }
  entity_save('cura_notification', $notification);
}

